@@ -25,6 +25,9 @@
 #ifndef CGAL_MESH_3_LLOYD_MOVE_H
 #define CGAL_MESH_3_LLOYD_MOVE_H
 
+#include <CGAL/license/Mesh_3.h>
+
+
 #include <CGAL/Mesh_3/config.h>
 
 #include <CGAL/convex_hull_2.h>
@@ -325,7 +328,8 @@ private:
     
     // Fit plane using point-based PCA: compute least square fitting plane
     Plane_3 plane;
-    CGAL::linear_least_squares_fitting_3(first,last,plane,Dimension_tag<0>());
+    Point_3 point;
+    CGAL::linear_least_squares_fitting_3(first,last,plane, point, Dimension_tag<0>(), Gt(),Default_diagonalize_traits<FT, 3>());
     
     // Project all points to the plane
     std::transform(first, last, first, Project_on_plane(plane));
