@@ -26,12 +26,16 @@
 #ifndef CGAL_MESH_DOMAIN_WITH_POLYLINE_FEATURES_3_H
 #define CGAL_MESH_DOMAIN_WITH_POLYLINE_FEATURES_3_H
 
+#include <CGAL/license/Mesh_3.h>
+
+
 #include <CGAL/iterator.h>
 #include <CGAL/enum.h>
 #include <CGAL/number_utils.h>
 #include <CGAL/AABB_tree.h>
 #include <CGAL/AABB_traits.h>
 #include <CGAL/is_streamable.h>
+#include <CGAL/Real_timer.h>
 
 #include <vector>
 #include <set>
@@ -95,6 +99,16 @@ public:
     return start_point() == end_point();
   }
 
+  /// Returns the angle at the first point.
+  /// \pre The polyline must be a cycle.
+  Angle angle_at_first_point() const {
+    CGAL_precondition(is_cycle());
+    const Point_3& first = points_.front();
+    const Point_3& next = points_[1];
+    const Point_3& prev = points_[points_.size() - 2];
+    return angle(prev, first, next);
+  }
+
   /// Returns the length of the polyline
   FT length() const
   {
@@ -380,30 +394,30 @@ struct Mesh_domain_segment_of_curve_primitive{
 
 template <typename MDwPF, bool patch_id_is_streamable>
 struct Display_incidences_to_patches_aux {
-  template <typename Container>
-  void operator()(std::ostream& os, typename MDwPF::Curve_segment_index id,
+  template <typename Container, typename Point>
+  void operator()(std::ostream& os, Point p, typename MDwPF::Curve_segment_index id,
                   const Container&) const;
 };
 
 template <typename MDwPF> //specialization when patch_id_is_streamable == false
 struct Display_incidences_to_patches_aux<MDwPF, false> {
-  template <typename Container>
-  void operator()(std::ostream& os,
+  template <typename Container, typename Point>
+  void operator()(std::ostream& os, Point p,
                   typename MDwPF::Curve_segment_index id,
                   const Container&) const;
 };
 
 template <typename MDwPF, bool curve_id_is_streamable>
 struct Display_incidences_to_curves_aux {
-  template <typename Container>
-  void operator()(std::ostream& os, typename MDwPF::Curve_segment_index id,
+  template <typename Container, typename Point>
+  void operator()(std::ostream& os, Point p, typename MDwPF::Curve_segment_index id,
                   const Container&) const;
 };
 
 template <typename MDwPF> //specialization when curve_id_is_streamable == false
 struct Display_incidences_to_curves_aux<MDwPF, false> {
-  template <typename Container>
-  void operator()(std::ostream& os, typename MDwPF::Curve_segment_index id,
+  template <typename Container, typename Point>
+  void operator()(std::ostream& os, Point p,  typename MDwPF::Curve_segment_index id,
                   const Container&) const;
 };
 
@@ -556,25 +570,34 @@ public:
                             IndicesOutputIterator out /*=
                                                         CGAL::Emptyset_iterator()*/);
 
+  template <typename InputIterator>
+  void
+  add_features(InputIterator first, InputIterator last)
+  { add_features(first, last, CGAL::Emptyset_iterator()); }
+
+  template <typename InputIterator>
+  void
+  add_features_with_context(InputIterator first, InputIterator last)
+  { add_features_with_context(first, last, CGAL::Emptyset_iterator()); }
+
   template <typename IndicesOutputIterator>
   IndicesOutputIterator
   get_incidences(Curve_segment_index id, IndicesOutputIterator out) const;
 
   template <typename IndicesOutputIterator>
   IndicesOutputIterator
-  get_corner_incidences(Curve_segment_index id, IndicesOutputIterator out) const;
+  get_corner_incidences(Corner_index id, IndicesOutputIterator out) const;
+
+  template <typename IndicesOutputIterator>
+  IndicesOutputIterator
+  get_corner_incident_curves(Corner_index id, IndicesOutputIterator out) const;
 
   typedef std::set<Surface_patch_index> Surface_patch_index_set;
 
   const Surface_patch_index_set&
   get_incidences(Curve_segment_index id) const;
 
-  void display_corner_incidences(std::ostream& os, Corner_index id);
-
-  template <typename InputIterator>
-  void
-  add_features(InputIterator first, InputIterator last)
-  { add_features(first, last, CGAL::Emptyset_iterator()); }
+  void display_corner_incidences(std::ostream& os, Point_3, Corner_index id);
 
   /// Insert one edge into domain
   /// InputIterator value type is Point_3
@@ -621,19 +644,33 @@ private:
 
 public:
   typedef CGAL::AABB_tree<AABB_curves_traits> Curves_AABB_tree;
+  typedef std::set<Surface_patch_index> Set_of_patch_ids;
+  typedef std::map<Point_3, Set_of_patch_ids> Corners_incidence_map;
 
 private:
+  Corners_incidence_map corners_incidence_map_;
   mutable Curves_AABB_tree curves_aabb_tree_;
   mutable bool curves_aabb_tree_is_built;
 
 public:
+  const Corners_incidence_map& corners_incidences_map() const
+  { return corners_incidence_map_; }
+
   const Curves_AABB_tree& curves_aabb_tree() const {
     if(!curves_aabb_tree_is_built) build_curves_aabb_tree();
     return curves_aabb_tree_;
   }
+  Curve_segment_index maximal_curve_segment_index() const {
+    if(edges_incidences_.empty()) return Curve_segment_index();
+    return boost::prior(edges_incidences_.end())->first;
+  }
 
   void build_curves_aabb_tree() const {
-    std::cerr << "Building curves AABB tree...\n";
+#if CGAL_MESH_3_VERBOSE
+    std::cerr << "Building curves AABB tree...";
+    CGAL::Real_timer timer;
+    timer.start();
+#endif
     curves_aabb_tree_.clear();
     for(typename Edges::const_iterator
           edges_it = edges_.begin(),
@@ -651,6 +688,10 @@ public:
     }
     curves_aabb_tree_.build();
     curves_aabb_tree_is_built = true;
+#if CGAL_MESH_3_VERBOSE
+    timer.stop();
+    std::cerr << " done (" << timer.time() * 1000 << " ms)" << std::endl;
+#endif
   } // end build_curves_aabb_tree()
 
 private:
@@ -788,6 +829,16 @@ add_features_with_context(InputIterator first, InputIterator last,
   // Insert one edge for each element
   for( ; first != last ; ++first )
   {
+    const typename Gt::Point_3& p1 = first->polyline_content.front();
+    const typename Gt::Point_3& p2 = first->polyline_content.back();
+    Set_of_patch_ids& ids_p1 = corners_incidence_map_[p1];
+    std::copy(first->context.adjacent_patches_ids.begin(),
+              first->context.adjacent_patches_ids.end(),
+              std::inserter(ids_p1, ids_p1.begin()));
+    Set_of_patch_ids& ids_p2 = corners_incidence_map_[p2];
+    std::copy(first->context.adjacent_patches_ids.begin(),
+              first->context.adjacent_patches_ids.end(),
+              std::inserter(ids_p2, ids_p2.begin()));
     Curve_segment_index curve_id =
       insert_edge(first->polyline_content.begin(), first->polyline_content.end());
     edges_incidences_[curve_id] = first->context.adjacent_patches_ids;
@@ -825,17 +876,31 @@ get_corner_incidences(Corner_index id,
   return std::copy(incidences.begin(), incidences.end(), indices_out);
 }
 
+template <class MD_>
+template <typename IndicesOutputIterator>
+IndicesOutputIterator
+Mesh_domain_with_polyline_features_3<MD_>::
+get_corner_incident_curves(Corner_index id,
+                           IndicesOutputIterator indices_out) const
+{
+  typename Corners_tmp_incidences::const_iterator it =
+    corners_tmp_incidences_.find(id);
+  const std::set<Curve_segment_index>& incidences = it->second;
+  return std::copy(incidences.begin(), incidences.end(), indices_out);
+}
+
 namespace internal { namespace Mesh_3 {
 
 template <typename MDwPF_, bool curve_id_is_streamable>
 // here 'curve_id_is_streamable' is true
-template <typename Container2>
+template <typename Container2, typename Point>
 void
 Display_incidences_to_curves_aux<MDwPF_,curve_id_is_streamable>::
-operator()(std::ostream& os, typename MDwPF_::Curve_segment_index id,
+operator()(std::ostream& os, Point p, typename MDwPF_::Curve_segment_index id,
            const Container2& corners_tmp_incidences_of_id) const
 {
-  os << "Corner #" << id << " is incident to the following curves: {";
+  os << "Corner #" << id << " (" << p
+     << ") is incident to the following curves: {";
   BOOST_FOREACH(typename MDwPF_::Curve_segment_index curve_index,
                 corners_tmp_incidences_of_id)
   {
@@ -846,26 +911,28 @@ operator()(std::ostream& os, typename MDwPF_::Curve_segment_index id,
 
 template <class MDwPF_>
 // here 'curve_id_is_streamable' is false
-template <typename Container2>
+template <typename Container2, typename Point>
 void
 Display_incidences_to_curves_aux<MDwPF_,false>::
-operator()(std::ostream& os, typename MDwPF_::Curve_segment_index id,
+operator()(std::ostream& os, Point p, typename MDwPF_::Curve_segment_index id,
            const Container2& corners_tmp_incidences_of_id) const
 {
-  os << "Corner #" << id << " is incident to "
+  os << "Corner #" << id << " (" << p
+     << ") is incident to "
      << corners_tmp_incidences_of_id .size()
      << " curve(s).\n";
 }
 
 template <typename MDwPF_, bool patch_id_is_streamable>
 // here 'patch_id_is_streamable' is true
-template <typename Container>
+template <typename Container, typename Point>
 void
 Display_incidences_to_patches_aux<MDwPF_,patch_id_is_streamable>::
-operator()(std::ostream& os, typename MDwPF_::Curve_segment_index id,
+operator()(std::ostream& os, Point p, typename MDwPF_::Curve_segment_index id,
            const Container& corners_incidences_of_id) const
 {
-  os << "Corner #" << id << " is incident to the following patches: {";
+  os << "Corner #" << id << " (" << p
+     << ") is incident to the following patches: {";
   BOOST_FOREACH(typename MDwPF_::Surface_patch_index i,
                 corners_incidences_of_id)
   {
@@ -876,13 +943,13 @@ operator()(std::ostream& os, typename MDwPF_::Curve_segment_index id,
 
 template <class MDwPF_>
 // here 'patch_id_is_streamable' is false
-template <typename Container>
+template <typename Container, typename Point>
 void
 Display_incidences_to_patches_aux<MDwPF_,false>::
-operator()(std::ostream& os, typename MDwPF_::Curve_segment_index id,
+operator()(std::ostream& os, Point p, typename MDwPF_::Curve_segment_index id,
            const Container& corners_incidences_id) const
 {
-  os << "Corner #" << id << " is incident to "
+  os << "Corner #" << id << " (" << p << ") is incident to "
      << corners_incidences_id.size()
      << " surface patch(es).\n";
 }
@@ -892,7 +959,7 @@ operator()(std::ostream& os, typename MDwPF_::Curve_segment_index id,
 template <class MD_>
 void
 Mesh_domain_with_polyline_features_3<MD_>::
-display_corner_incidences(std::ostream& os, Corner_index id)
+display_corner_incidences(std::ostream& os, Point_3 p, Corner_index id)
 {
   typedef Mesh_domain_with_polyline_features_3<MD_> Mdwpf;
   typedef is_streamable<Surface_patch_index> i_s_spi;
@@ -901,8 +968,8 @@ display_corner_incidences(std::ostream& os, Corner_index id)
   using namespace internal::Mesh_3;
   typedef Display_incidences_to_curves_aux<Mdwpf,i_s_csi::value> D_i_t_c;
   typedef Display_incidences_to_patches_aux<Mdwpf,i_s_spi::value> D_i_t_p;
-  D_i_t_c()(os, id, corners_tmp_incidences_[id]);
-  D_i_t_p()(os, id, corners_incidences_[id]);
+  D_i_t_c()(os, p, id, corners_tmp_incidences_[id]);
+  D_i_t_p()(os, p, id, corners_incidences_[id]);
 }
 
 template <class MD_>
@@ -920,14 +987,20 @@ compute_corners_incidences()
       corner_tmp_incidences = corners_tmp_incidences_[id];
 
     // If the corner is incident to only one curve, and that curve is a
-    // cycle, then remove the corner from the set.
+    // cycle, then remove the corner from the set, only if the angle is not
+    // acute. If the angle is acute, the corner must remain as a corner,
+    // to deal correctly with the angle.
     if(corner_tmp_incidences.size() == 1 &&
        is_cycle(Point_3(), *corner_tmp_incidences.begin()))
     {
-      typename Corners::iterator to_erase = cit;
-      ++cit;
-      corners_.erase(to_erase);
-      continue;
+      const Curve_segment_index curve_id = *corner_tmp_incidences.begin();
+      const Polyline& polyline = edges_[curve_id];
+      if(polyline.angle_at_first_point() == OBTUSE) {
+        typename Corners::iterator to_erase = cit;
+        ++cit;
+        corners_.erase(to_erase);
+        continue;
+      }
     }
 
     Surface_patch_index_set& incidences = corners_incidences_[id];
@@ -940,7 +1013,7 @@ compute_corners_incidences()
                                    incidences.begin()));
     }
 #ifdef CGAL_MESH_3_PROTECTION_DEBUG
-    display_corner_incidences(std::cerr, id);
+    display_corner_incidences(std::cerr, cit->first, id);
 #endif // CGAL_MESH_3_PROTECTION_DEBUG
 
     // increment the loop variable
