@@ -20,6 +20,9 @@
 #ifndef CGAL_UPSAMPLE_POINT_SET_H
 #define CGAL_UPSAMPLE_POINT_SET_H
 
+#include <CGAL/license/Point_set_processing_3.h>
+
+
 #include <CGAL/property_map.h>
 #include <CGAL/point_set_processing_assertions.h>
 #include <CGAL/internal/Rich_grid.h>
@@ -31,12 +34,6 @@
 #include <set>
 #include <utility>
 
-#include <boost/version.hpp>
-#if BOOST_VERSION >= 104000
-  #include <boost/property_map/property_map.hpp>
-#else
-  #include <boost/property_map.hpp>
-#endif
 
 //#define  CGAL_PSP3_VERBOSE
 
@@ -388,13 +385,8 @@ edge_aware_upsample_point_set(
   ForwardIterator it = first; // point iterator
   for(unsigned int i = 0; it != beyond; ++it, ++i)
   {
-#ifdef CGAL_USE_PROPERTY_MAPS_API_V1
-    rich_point_set[i].pt = get(point_pmap, it);
-    rich_point_set[i].normal = get(normal_pmap, it);
-#else
     rich_point_set[i].pt = get(point_pmap, *it);
     rich_point_set[i].normal = get(normal_pmap, *it);
-#endif
 
     rich_point_set[i].index = i;
     bbox += rich_point_set[i].pt.bbox();
